var Chart = function(options, request) {
	var self = this;
	this.type = options.type;
	this.container = options.container;
	this.buildTable = function() {
		function parseDate(date) {
			var newDate = moment(date);
			var formattedDate = newDate.format("MM/DD/YY @ HH:MM");
			return formattedDate;
		}

		var d = options.table.data,
			$table = document.getElementById(options.table.container);
		var tableHTML = '<thead><tr><th>Date/Time</th><th>Service Description</th></tr></thead><tbody>';
		for (var i = 0; i < d.length; i++) {
			d[i].date_time_formatted = parseDate(d[i].date_time);
			tableHTML += '<tr><td>'+d[i].date_time_formatted+'</td><td>'+d[i].station_svc_descrip+'</td></tr>';
		}
		tableHTML += '</tbody>';
		$table.innerHTML = tableHTML;

	};
	this.buildChart = function() {
		google.charts.load('current', {'packages':['corechart', 'controls', 'gauge']});
		google.charts.setOnLoadCallback(drawChart);

		function pageResize() {
			var body = document.body,
				html = document.documentElement;

			var height = Math.max( body.scrollHeight, body.offsetHeight, 
               html.clientHeight, html.scrollHeight, html.offsetHeight );
			var leftSidebar = document.getElementsByClassName('left-sidebar')[0];
			leftSidebar.style.height = height + 'px';
		}

		// process null value
		function processNull(data) {
			if (data == null ) {
				return null;
			} else {
				return +data;
			}
		}

		function drawChart() {
			
			var data  = new google.visualization.DataTable();
	
			if (self.type == 1) { // line chart
				// create shorthand names for field labels/types
				var fields = options.fields,
					fl = options.fieldLabels,
					ft = options.fieldTypes;

				// loop through field labels and types
				// adding columns to table accordingly
				ft.forEach(function(d,i) {
					data.addColumn(d, fl[i]);
				})

				// find the index of the date field
				var dateFieldIndex = ft.indexOf("datetime"),
					dateFieldName = fields[dateFieldIndex];

				// convert the date/time to a javascript date using the column index
				// for the date/time field
				for (var i = 0; i < request.length; i++) {
					request[i].date_time_formatted = new Date(request[i][dateFieldName]);
					data.addRows([[request[i].date_time_formatted, +request[i].location_ndx,request[i].well_selector, processNull(request[i].pressure_hourly)]]);
				}

				var lineChartOptions = {
					width: '100%',
					height: 460,
					legend: {
						position: 'top'
					}
				}

				var lineChart = new google.visualization.ChartWrapper({
					chartType: 'LineChart',
					containerId: self.container,
					options: lineChartOptions,
					dataTable: data,
					view: {'columns': [0,3]}
				})

				google.visualization.events.addListener(lineChart, 'ready', pageResize);	 
				lineChart.draw();

			}

			if (self.type == 2) { // scatter chart
				// create shorthand names for field labels/types
				var fields = options.fields,
					fl = options.fieldLabels,
					ft = options.fieldTypes;

				// loop through field labels and types
				// adding columns to table accordingly
				ft.forEach(function(d,i) {
					data.addColumn(d, fl[i]);
				})

				// find the index of the date field
				var dateFieldIndex = ft.indexOf("datetime"),
					dateFieldName = fields[dateFieldIndex];

				// convert the date/time to a javascript date using the column index
				// for the date/time field
				for (var i = 0; i < request.length; i++) {
					request[i].date_time_formatted = new Date(request[i][dateFieldName]);
					data.addRows([[request[i].date_time_formatted, +request[i].location_ndx,request[i].well_selector, processNull(request[i].pressure_hourly)]]);
				}

				var scatterChartOptions = {
					width: '100%',
					height: 460,
					legend: {
						position: 'top'
					}
				}

				var scatterChart = new google.visualization.ChartWrapper({
					chartType: 'ScatterChart',
					containerId: self.container,
					options: scatterChartOptions,
					dataTable: data,
					view: {'columns': [0,3]}
				})
	 
				scatterChart.draw();
			}

			if (self.type == 3) { // Area/Line Combo Chart

				// create shorthand names for field labels/types
				var fields = options.fields,
					fl = options.fieldLabels,
					ft = options.fieldTypes;

				// loop through field labels and types
				// adding columns to table accordingly
				ft.forEach(function(d,i) {
					data.addColumn(d, fl[i]);
				})

				// find the index of the date field
				var dateFieldIndex = ft.indexOf("datetime"),
					dateFieldName = fields[dateFieldIndex];

				// convert the date/time to a javascript date using the column index
				// for the date/time field
				for (var i = 0; i < request.length; i++) {
					request[i].date_time_formatted = new Date(request[i][dateFieldName]);
					data.addRows([[request[i].date_time_formatted, +request[i].location_ndx,request[i].well_selector, processNull(request[i].dtw_hourly), processNull(request[i].pumping_hourly), processNull(request[i].depth_gw_to_pump_hourly)]]);
				}

				data.sort({column: 0, desc:true});
				dateformatter = new google.visualization.DateFormat({pattern: "MMM d, h:mm a"});
				dateformatter.format(data, 0);
				var view = new google.visualization.DataView(data);

				var comboChartOptions = {
					width: '100%',
					height: 460,
					isStacked: true,
					chartArea: {
      					backgroundColor: '#f5f5f5'
    				},
					series: {
      					0: {
      							type: 'area', 
      							color: '#2b8cbe', 
      							targetAxisIndex: 0, 
      							lineWidth:1,
      							areaOpacity:0,
      							visibleInLegend: false,
      							//tooltip: false

      					},
      					1: {
      							type: 'line', 
      							targetAxisIndex: 1, 
      							color: '#e6550d', 
      							pointSize: 0, 
      							lineWidth: 2
      					},
      					2: {
      							type: 'area', 
      							targetAxisIndex: 0, 
      							color: '#2b8cbe', 
      							lineWidth:0,
      							areaOpacity:0.2,
      							tooltip: false
      					}
      				},
      				vAxes: {
      					1: {title:'Well Pumping, gpm', format: '#'},
      					0: {title:'Depth to Water Table, ft', format: '#.##'}
    				},
					focusTarget: 'category'
				}

				var comboChart = new google.visualization.ChartWrapper({
					chartType: 'ComboChart',
					containerId: self.container,
					options: comboChartOptions,
					dataTable: data,
					view: {'columns': [0,3,4,5]}
				})

				google.visualization.events.addListener(comboChart, 'ready', pageResize);	 
				comboChart.draw();
			}

			if (self.type == 4) { // Column/Line Combo Chart
				// create shorthand names for field labels/types
				var fields = options.fields,
					fl = options.fieldLabels,
					ft = options.fieldTypes;

				// loop through field labels and types
				// adding columns to table accordingly
				ft.forEach(function(d,i) {
					data.addColumn(d, fl[i]);
				});

				// find the index of the date field
				var dateFieldIndex = ft.indexOf("datetime"),
					dateFieldName = fields[dateFieldIndex];

				// convert the date/time to a javascript date using the column index
				// for the date/time field
				for (var i = 0; i < request.length; i++) {
					request[i].date_time_formatted = new Date(request[i][dateFieldName]);
					data.addRows([[request[i].date_time_formatted, +request[i].location_ndx,request[i].well_selector, processNull(request[i].daily_volume_bbl), processNull(request[i].cumulative)]]);
					//console.log([request[i][dateFieldName],request[i].date_time_formatted])
				}

				data.sort({column: 0, desc:true});
				dateformatter=new google.visualization.DateFormat({pattern: "MMM d"});
				dateformatter.format(data, 0);
				var view = new google.visualization.DataView(data);


				var comboColumnChartOptions = {
					width: '100%',
					height: 460,
					chartArea: {
      					backgroundColor: '#f5f5f5'
    				},
					series: {
      					0: {
      							type: 'bars', 
      							color: '#31a354', 
      							targetAxisIndex: 0
      					},
      					1: {
      							type: 'line', 
      							targetAxisIndex: 0, 
      							color: '#e6550d', 
      							pointSize: 0, 
      							lineWidth: 2
      					}
      				},
      				legend: {position: 'top'},
      				vAxes: {
      					0: {title:'Volume Pumped, bbl', format: 'decimal'}
    				},
    				focusTarget: 'category'
				}

				var comboColumnChart = new google.visualization.ChartWrapper({
					chartType: 'ComboChart',
					containerId: self.container,
					options: comboColumnChartOptions,
					dataTable: data,
					view: {'columns': [0,3,4]}
				})

				google.visualization.events.addListener(comboColumnChart, 'ready', pageResize);
	 
				comboColumnChart.draw();
			} // if self.type == 4
			if (self.type == 5) { // gauge chart
				// create shorthand names for field labels/types
				var fields = options.fields,
					fl = options.fieldLabels,
					ft = options.fieldTypes,
					gaugeRanges = options.gaugeRanges;

				// loop through field labels and types
				// adding columns to table accordingly
				ft.forEach(function(d,i) {
					data.addColumn(d, fl[i]);
				});

				for (var i = 0; i < request.length; i++) {
					data.addRows([[request[i].descrip, processNull(request[i].hrs_since_service)]]);
				}

				data.setColumnLabel(0,'Desc');
			    data.setColumnLabel(1,'Hours');
				var view = new google.visualization.DataView(data);

			    // define chart options
			    var gaugeOptions = {
			      width: '100%', 
			      height: 200,
			      redFrom: +gaugeRanges[0].notification2_hrs, 
			      redTo: +gaugeRanges[0].service_due_hrs,
			      yellowFrom: +gaugeRanges[0].notification1_hrs, 
			      yellowTo: +gaugeRanges[0].notification2_hrs,
			      minorTicks: 500,
			      max: +gaugeRanges[0].service_due_hrs
			    };

			    // create gauge chart
			    var gaugeChart = new google.visualization.Gauge(document.getElementById('gauge-chart'));
			    google.visualization.events.addListener(gaugeChart, 'ready', pageResize);
			    gaugeChart.draw(data, gaugeOptions);
			} // if self.type == 5
		}
	};
	this.init = (function() {
		self.buildChart();
		if (typeof options.table !== "undefined") {
			self.buildTable();
		}
	})();
}